import argparse
from .organizer import Organizer


def main():

    parser = argparse.ArgumentParser(
        prog="pythod",
        description="Load CLI flags for the Pythod organizer.",
        usage="pythod --dir /absolute/path/to/directory/for/organizing"
    )

    parser.add_argument(
        "--config",
        nargs="?",
        type=str,
        help="Path to your custom configuration file. If no passed, will load the default one.",
    )

    parser.add_argument(
        "--dir",
        type=str,
        help="Path to the directory that will be organized.",
        required=True
    )

    cli_arguments = parser.parse_args()

    if cli_arguments.config == None:
        print('>>> Default configuration will be loaded.')

    organizer = Organizer(cli_arguments.dir, cli_arguments.config)
    organizer.organize()



from typing import List
from .content import Content


class Configuration(object):
    contents: List[Content]

    def __init__(self, contents: List[Content]):
        self.contents = contents

    def get_contents(self):
        return self.contents

    def __str__(self) -> str:
        contents_layout = ''

        for content in self.contents:
            contents_layout += 'Content class: {}\nTarget folder: {}\nContent class indicators:\n'.format(
                content.get_content_class(),
                content.get_target_directory()
            )

            for indicator in content.get_indicators():
                contents_layout += '\t{}\n'.format(indicator)
            contents_layout += '-'*30+'\n'
        return contents_layout



__all__ = [
    'configuration_deserializer',
    'configuration',
    'content',
    'organizer',
    'main'
]
from .configuration import Configuration
from .configuration_deserializer import ConfigurationDeserializer
from .content import Content
from .organizer import Organizer
from .main import main



from .configuration_deserializer import ConfigurationDeserializer
import os
import shutil


class Organizer():
    def __init__(self, org_directory, configuration_path=None):
        self.__org_directory = org_directory
        self.__configuration_path = configuration_path

    def organize(self):
        configuration = self.__load_configuration()
        self.__build_target_directories(configuration)

        for root, dirs, files in os.walk(self.__org_directory):
            for _dir in dirs:
                target_directory = self.__get_target_directory(
                    configuration,
                    _dir,
                    True
                )

                if target_directory != None:
                    self.__move_object(
                        os.path.join(root, _dir),
                        os.path.join(root, target_directory)
                    )

            for _file in files:
                target_directory = self.__get_target_directory(
                    configuration,
                    _file
                )

                if target_directory != None:
                    self.__move_object(
                        os.path.join(root, _file),
                        os.path.join(root, target_directory)
                    )

            break

    def __move_object(self, source, target):
        shutil.move(source, target)

    def __get_target_directory(self, configuration, element, is_directory=False):
        for content in configuration.contents:
            if not is_directory and element.endswith(content.get_indicators()):
                return content.get_target_directory()
            elif is_directory and any(string in element for string in content.get_indicators()):
                print(content.get_indicators())
                return content.get_target_directory()

        print('>>> Target directory not found for: {}'.format(element))
        return None

    def __build_target_directories(self, configuration):
        for content in configuration.contents:

            print(os.path.join(self.__org_directory,
                               content.get_target_directory()))
            if not os.path.isdir(os.path.join(self.__org_directory, content.get_target_directory())):
                os.mkdir(os.path.join(
                    self.__org_directory,
                    content.get_target_directory()
                ))

                print('>>> Created content directory {}.'.format(
                    content.get_target_directory()
                ))

    def __load_configuration(self):
        if self.__configuration_path == None:
            current_dir = self.__get_current_directory_path()

            self.__configuration_path = current_dir + '/config.json'

        deserializer = ConfigurationDeserializer(self.__configuration_path)
        configuration = deserializer.deserialize()

        return configuration

    def __get_current_directory_path(self):
        return os.path.dirname(os.path.abspath(__file__))



from typing import Tuple


class Content(object):
    content_class: str
    target_directory: str
    indicators: Tuple

    def __init__(self, content_class: str, indicators: Tuple, target_directory: str):
        self.content_class = content_class
        self.indicators = indicators
        self.target_directory = target_directory

    def get_content_class(self):
        return self.content_class

    def get_target_directory(self):
        return self.target_directory

    def get_indicators(self):
        return self.indicators



import json
from .configuration import Configuration
from .content import Content


class ConfigurationDeserializer(object):
    def __init__(self, configuration_file_location):
        self.__configuration_file_location = configuration_file_location

    def deserialize(self) -> Configuration:
        configuration_file_contents = self.__read_configuration_file()
        configuration_json = json.loads(configuration_file_contents)

        configuration = self.__decode_configuration(configuration_json)

        return configuration

    def __decode_configuration(self, configuration_json) -> Configuration:
        contents = list()
        if 'contents' in configuration_json:
            for content in configuration_json['contents']:
                _content = Content(
                    content['content_class'],
                    tuple(content['indicators']),
                    content['target_directory']
                )

                contents.append(_content)

        return Configuration(contents)

    def __read_configuration_file(self) -> str:
        with open(self.__configuration_file_location) as configuration_file:
            return configuration_file.read()



import setuptools

with open('README.md') as readme:
    long_description = readme.read()

setuptools.setup(
    name='pythod',
    version='0.0.5',
    author='Ivan Zlatanov',
    author_email='i_zlatanov@protonmail.com',
    description='Pythod is a lightweigh Python directory organizing CLI tool.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/not-so-cool-anymore/pythod',
    package_dir={
        'pythod': 'pythod'
    },
    package_data={
        '': ['config.json']
    },
    include_package_data=True,
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'pythod = pythod.main:main',
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5'
)
